/*
 * Amanda, The Advanced Maryland Automatic Network Disk Archiver
 * Copyright (c) 1991 University of Maryland
 * All Rights Reserved.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of U.M. not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  U.M. makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * U.M. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL U.M.
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: James da Silva, Systems Design and Analysis Group
 *			   Computer Science Department
 *			   University of Maryland at College Park
 */
/*
 * driver.h - defines and globals for the Amanda driver
 */
#define MAX_DUMPERS 15
#define MAX_ARGS 10
#define MAX_LINE 1024

#ifndef GLOBAL
#define GLOBAL extern
#endif

/* define schedule structure */

typedef struct sched_s {
    int est_kps, attempted, level, est_time, degr_level, degr_time;
    unsigned long est_size, degr_size, act_size;
    char destname[128];				/* file/port name */
    struct dumper_s *dumper;
} sched_t;

#define sched(dp)	((sched_t *) (dp)->up)

/* dumper process structure */

typedef struct dumper_s {
    int busy, pid;
    int infd, outfd;
    disk_t *dp;
} dumper_t;

GLOBAL dumper_t dmptable[MAX_DUMPERS];
GLOBAL int inparallel, big_dumpers;
GLOBAL int degraded_mode;

/* command/result tokens */

typedef enum {
    BOGUS, QUIT, DONE,
    FILE_DUMP, PORT_DUMP, CONTINUE, ABORT,		/* dumper cmds */
    FAILED, TRYAGAIN, NO_ROOM, ABORT_FINISHED,		/* dumper results */
    FATAL_TRYAGAIN,
    START_TAPE, FILE_WRITE, PORT_WRITE, 		/* taper cmds */
    PORT, TAPE_ERROR, TAPE_OK,				/* taper results */
    LAST_TOK
} tok_t;

extern char *cmdstr[];
extern char *pname;

GLOBAL tok_t tok;
GLOBAL fd_set readset;
GLOBAL disklist_t waitq, runq, stoppedq, tapeq;
GLOBAL int pending_aborts, inside_dump_to_tape;
GLOBAL int total_bandwidth, allocated_bandwidth;
GLOBAL int total_diskspace, allocated_diskspace;
GLOBAL int verbose;

GLOBAL int taper, taper_busy, taper_pid;
GLOBAL disk_t *taper_disk;

GLOBAL int argc;
GLOBAL char *argv[MAX_ARGS];
GLOBAL int maxfd;
GLOBAL int force_parameters, use_lffo;
GLOBAL char datestamp[80], taper_program[80], dumper_program[80];
GLOBAL char *diskdir;

/* local functions */
void startup_dump_process P((dumper_t *dumper));
void startup_dump_processes P((void));
void startup_tape_process P((void));
void taper_cmd P((tok_t cmd, void *ptr));
void dumper_cmd P((dumper_t *dumper, tok_t cmd, disk_t *dp));
tok_t getresult P((int fd));
int start_some_dumps P((disklist_t *rq));
int some_dumps_in_progress P((void));
int dump_to_tape P((disk_t *dp));

int free_space P((void));
int free_kps P((void));
void allocate_bandwidth P((int kps));
void deallocate_bandwidth P((int kps));
void allocate_diskspace P((int size));
void deallocate_diskspace P((int size));

void force_incrementals P((disklist_t *q));
void handle_taper_result P((void));
void handle_dumper_result P((int fd));
dumper_t *idle_dumper P((void));
disklist_t read_schedule P((disklist_t *wq));
void construct_datestamp P((char *buf));

void dump_state P((char *str));
void short_dump_state P((void));

disk_t *serial2disk P((char *str));
char *disk2serial P((disk_t *dp));
void free_serial P((char *str));
